// -*-C++-*-
/*!
 * @file  CameraInfoServiceSVC_impl.h
 * @brief Service implementation header of CameraInfoService.idl
 *
 */

#include "CameraInfoServiceSkel.h"

#ifndef CAMERAINFOSERVICESVC_IMPL_H
#define CAMERAINFOSERVICESVC_IMPL_H

#include <rtm/Manager.h> // for using 'usleep()' function

#include <string>
using namespace std;
/*!
 * @class CameraInfoServiceSVC_impl
 * Example class implementing IDL interface CameraInfoService
 */
class CameraInfoServiceSVC_impl
 : public virtual POA_CameraInfoService,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~CameraInfoServiceSVC_impl();

 public:
  /*!
   * @brief standard constructor
   */
   CameraInfoServiceSVC_impl();
  /*!
   * @brief destructor
   */
   virtual ~CameraInfoServiceSVC_impl();

   // attributes and operations
   CORBA::Boolean getIntrinsicParameter(CameraInfoService::IntrinsicParameter data)
	   throw (CORBA::SystemException);
   CORBA::Boolean getDistortionParameter(CameraInfoService::DistortionParameter data)
	   throw (CORBA::SystemException);

private:
	// camera informations
	// intrinsic parameter
	double _intrinsic[6];
	// distortion parameter
	double _distortion[2];
public:
	// set camera informations
	inline bool setIntrinsicParameter(double intrinsic[6]){
		memcpy(_intrinsic, intrinsic, sizeof(double)*6);
		return true;
	}
	inline bool setDistortionParameter(double distortion[2]){
		memcpy(_distortion, distortion, sizeof(double)*2);
		return true;
	}

private:
	// lock flag
	// true(lock) / false(unlock)
	bool _getIntrinsicLock;
	bool _getDistortionLock;
public:
	// lock/unlock service port
	void lock_getIntrinsicService(){_getIntrinsicLock = true;}
	void unlock_getIntrinsicService(){_getIntrinsicLock = false;}
	void lock_getDistortionService(){_getDistortionLock = true;}
	void unlock_getDistortionService(){_getDistortionLock = false;}
};


/*!
 * @class StereoCameraInfoServiceSVC_impl
 * Example class implementing IDL interface StereoCameraInfoService
 */
class StereoCameraInfoServiceSVC_impl
 : public virtual POA_StereoCameraInfoService,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~StereoCameraInfoServiceSVC_impl();

 public:
  /*!
   * @brief standard constructor
   */
   StereoCameraInfoServiceSVC_impl();
  /*!
   * @brief destructor
   */
   virtual ~StereoCameraInfoServiceSVC_impl();

   // attributes and operations
	// get extrinsic parameter between cameras
   CORBA::Boolean getExtrinsicParameterBC( StereoCameraInfoService::ExtrinsicParameter data)
		throw(CORBA::SystemException);

private:
	// camera informations
	// extrinsic parameter
	double _extrinsic[12];
public:
	// set camera informations
	inline bool setExtrinsicParameter(double extrinsic[12]){
		memcpy(_extrinsic, extrinsic, sizeof(double)*12);
		return true;
	}
	inline bool setBaseLine(double baseline){
		_extrinsic[0] = 1.0; _extrinsic[1] = 0.0; _extrinsic[2] = 0.0; _extrinsic[3] = 0.0;
		_extrinsic[4] = 0.0; _extrinsic[5] = 1.0; _extrinsic[6] = 0.0; _extrinsic[7] = baseline;
		_extrinsic[9] = 0.0; _extrinsic[9] = 0.0; _extrinsic[10] = 1.0; _extrinsic[11] = 0.0;
 		return true;
	}

private:
	// lock flag
	// true(lock) / false(unlock)
	bool _getExtrinsicLock;
public:
	// lock/unlock service port
	void lock_getExtrinsicService(){_getExtrinsicLock = true;}
	void unlock_getExtrinsicService(){_getExtrinsicLock = false;}
};

#endif // CAMERAINFOSERVICESVC_IMPL_H


